@@ -1,5 +1,6 @@
 import collections
 import json
+from math import ceil
 from typing import Generator, Tuple, cast
 
 import numpy as np
@@ -41,6 +42,8 @@ class Fit(collections.abc.Mapping):
             constrained_param_names,
         )
         self.num_warmup, self.num_samples = num_warmup, num_samples
+        if not isinstance(num_thin, int):
+            raise ValueError(f"{type(num_thin)} object cannot be interpreted as an integer: num_thin={num_thin}")
         self.num_thin, self.save_warmup = num_thin, save_warmup
 
         # `self.sample_and_sampler_param_names` collects the sample and sampler param names.
@@ -51,7 +54,9 @@ class Fit(collections.abc.Mapping):
 
         num_flat_params = sum(np.product(dims_ or 1) for dims_ in dims)  # if dims == [] then it is a scalar
         assert num_flat_params == len(constrained_param_names)
-        num_samples_saved = (self.num_samples + self.num_warmup * self.save_warmup) // self.num_thin
+        num_samples_saved = ceil(self.num_samples / self.num_thin) + ceil(
+            (self.num_warmup * self.save_warmup) / self.num_thin
+        )
 
         # self._draws holds all the draws. We cannot allocate it before looking at the draws
         # because we do not know how many sampler-specific parameters are present. Later in this
@@ -69,29 +74,33 @@ class Fit(collections.abc.Mapping):
                 except ValueError:
                     # Occurs when draws contain an nan or infinity. simdjson cannot parse such values.
                     msg = json.loads(line)
-                if msg["topic"] == "sample":
-                    # Ignore sample message which is mixed together with proper draws.
-                    if not isinstance(msg["values"], (simdjson.Object, dict)):
-                        continue
+                try:
+                    if msg["topic"] == "sample":
+                        # Ignore sample message which is mixed together with proper draws.
+                        if not isinstance(msg["values"], (simdjson.Object, dict)):
+                            continue
 
-                    # for the first draw: collect sample and sampler parameter names.
-                    if not hasattr(self, "_draws"):
-                        feature_names = cast(Tuple[str, ...], tuple(msg["values"].keys()))
-                        self.sample_and_sampler_param_names = tuple(
-                            name for name in feature_names if name.endswith("__")
-                        )
-                        num_rows = len(self.sample_and_sampler_param_names) + num_flat_params
-                        # column-major order ("F") aligns with how the draws are stored (in cols).
-                        self._draws = np.empty((num_rows, num_samples_saved, num_chains), order="F")
-                        # rudimentary check of parameter order (sample & sampler params must be first)
-                        if num_flat_params and feature_names[-1].endswith("__"):
-                            raise RuntimeError(
-                                f"Expected last parameter name to be one declared in program code, found `{feature_names[-1]}`"
+                        # for the first draw: collect sample and sampler parameter names.
+                        if not hasattr(self, "_draws"):
+                            feature_names = cast(Tuple[str, ...], tuple(msg["values"].keys()))
+                            self.sample_and_sampler_param_names = tuple(
+                                name for name in feature_names if name.endswith("__")
                             )
+                            num_rows = len(self.sample_and_sampler_param_names) + num_flat_params
+                            # column-major order ("F") aligns with how the draws are stored (in cols).
+                            self._draws = np.empty((num_rows, num_samples_saved, num_chains), order="F")
+                            # rudimentary check of parameter order (sample & sampler params must be first)
+                            if num_flat_params and feature_names[-1].endswith("__"):
+                                raise RuntimeError(
+                                    f"Expected last parameter name to be one declared in program code, found `{feature_names[-1]}`"
+                                )
 
-                    draw_row = tuple(msg["values"].values())  # a "row" of values from a single draw from Stan C++
-                    self._draws[:, draw_index, chain_index] = draw_row
-                    draw_index += 1
+                        draw_row = tuple(msg["values"].values())  # a "row" of values from a single draw from Stan C++
+                        self._draws[:, draw_index, chain_index] = draw_row
+                        draw_index += 1
+                finally:
+                    # clean up `Object`s produced by parser, required by simdjson
+                    del msg
             assert draw_index == num_samples_saved
         assert self.sample_and_sampler_param_names and self._draws.size
         self._draws.flags["WRITEABLE"] = False  # type: ignore
@@ -124,7 +133,7 @@ class Fit(collections.abc.Mapping):
         param_indexes = self._parameter_indexes(param)
         param_dim = [] if param in self.sample_and_sampler_param_names else self.dims[self.param_names.index(param)]
         # fmt: off
-        num_samples_saved = (self.num_samples + self.num_warmup * self.save_warmup) // self.num_thin
+        num_samples_saved = ceil(self.num_samples / self.num_thin) + ceil((self.num_warmup * self.save_warmup) / self.num_thin)
         assert self._draws.shape == (len(self.sample_and_sampler_param_names) + len(self.constrained_param_names), num_samples_saved, self.num_chains)
         # fmt: on
         if not len(param_indexes):
